"""Fuels Enumeration"""

# -- Boiler Fuels
NATURAL_GAS = 'NATURAL_GAS'
OIL = 'OIL'
WOOD_LOG = 'WOOD_LOG'
WOOD_PELLET = 'WOOD_PELLET'
ELECTRICITY = 'ELECTRICITY'

# -- District Energy Carriers
HARD_COAL_CGS_70_PHC = 'HARD_COAL_CGS_70_PHC'
HARD_COAL_CGS_35_PHC = 'HARD_COAL_CGS_35_PHC'
HARD_COAL_HS_0_PHC = 'HARD_COAL_HS_0_PHC'
GAS_CGS_70_PHC = 'GAS_CGS_70_PHC'
GAS_CGS_35_PHC = 'GAS_CGS_35_PHC'
GAS_HS_0_PHC = 'GAS_HS_0_PHC'
OIL_CGS_70_PHC = 'OIL_CGS_70_PHC'
OIL_CGS_35_PHC = 'OIL_CGS_35_PHC'
OIL_HS_0_PHC = 'OIL_HS_0_PHC'
